package zero1hd.rhythmbullet.desktop.audio.visualizer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.VertexAttribute;
import com.badlogic.gdx.graphics.VertexAttributes.Usage;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.RhythmBullet;

public class CircularVisualizer extends Visualizer implements Disposable {
	private int indexCirclePoints;
	private int centerX, centerY;
	private int r;
	private int componentCount = 2 + 1;
	private float[][] circlePoints;
	private float[] vertComponents;
	private float color;
	private boolean flushed;
	private Mesh mesh;
	private ShaderProgram shader;
	private int barCount = 180;
	private float barHeightMultiplier = 1.5f;
	private float[] audioSpectrum;
	private Camera camera;
	public CircularVisualizer() {
		shader = new ShaderProgram(Gdx.files.internal("shaders/mesh.vsh"), Gdx.files.internal("shaders/mesh.fsh"));
		r = RhythmBullet.pixels_per_unit*2;
		System.out.println(RhythmBullet.pixels_per_unit);
	}
	
	/**
	 * Only should be called when all changes have been done.
	 */
	public void applyPositionChanges() {
		int vertsReq = calculateVerticeCount(centerX, centerY, r);
		vertComponents = new float[vertsReq * componentCount];
		circlePoints = new float[vertsReq][2];
		
		if (mesh != null) {
			mesh.dispose();
		}
		createCircleVertices(centerX, centerY, r);
		mesh = new Mesh(false, circlePoints.length, 0, new VertexAttribute(Usage.Position, 2, "a_position"), new VertexAttribute(Usage.ColorPacked, 4, "a_color"));
	}
	
	public void setCenter(int x, int y) {
		this.centerX = x;
		this.centerY = y;
	}
	
	private void addCircleVertex(int x, int y) {
		circlePoints[indexCirclePoints][0] = x;
		circlePoints[indexCirclePoints][1] = y;
		indexCirclePoints++;
	}
	
	public void drawCircle() {
		if (flushed) {
			for (int circleVertexID = 0; circleVertexID < vertComponents.length; circleVertexID++) {
				vertComponents[circleVertexID] = circlePoints[circleVertexID][0];
				circleVertexID++;
				vertComponents[circleVertexID] = circlePoints[circleVertexID][1];
				circleVertexID++;
				vertComponents[circleVertexID] = color;
			}
			flushed = false;
		} else {
			flush();
		}
	}
	
	public void flush() {
		if (!flushed) {
			mesh.setVertices(vertComponents);
			
			Gdx.gl.glEnable(GL20.GL_BLEND);
			Gdx.gl.glBlendFunc(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);
			
			int vertexCount = circlePoints.length;
			
			shader.begin();
			shader.setUniformMatrix("u_projTrans", camera.combined);
			
			mesh.render(shader, GL20.GL_TRIANGLES, 0, vertexCount);
			shader.end();
			
			flushed = true;
			Gdx.gl.glDepthMask(true);
		}
	}
	
	private void createCircleVertices(int centerX, int centerY, int radius) {
		int x = radius - 1;
		int y = 0;
		int dx = 1;
		int dy = 1;
		int err = dx - (radius << 1);
		indexCirclePoints = 0;
	    while (x >= y)
	    {
	        addCircleVertex(centerX + x, centerY + y);
	        addCircleVertex(centerX + y, centerY + x);
	        addCircleVertex(centerX - y, centerY + x);
	        addCircleVertex(centerX - x, centerY + y);
	        addCircleVertex(centerX - x, centerY - y);
	        addCircleVertex(centerX - y, centerY - x);
	        addCircleVertex(centerX + y, centerY - x);
	        addCircleVertex(centerX + x, centerY - y);

	        if (err <= 0) {
	            y++;
	            err += dy;
	            dy += 2;
	        } else {
	            x--;
	            dx += 2;
	            err += dx - (radius << 1);
	        }
	    }
	}
	
	private int calculateVerticeCount(int centerX, int centerY, int radius) {
		int x = radius - 1;
		int y = 0;
		int dx = 1;
		int dy = 1;
		int err = dx - (radius << 1);
		int vertCount = 0;
	    while (x >= y)
	    {
	    	vertCount += 8;

	        if (err <= 0) {
	            y++;
	            err += dy;
	            dy += 2;
	        } else {
	            x--;
	            dx += 2;
	            err += dx - (radius << 1);
	        }
	    }
	    
	    return vertCount;
	}
	
	public void setColor(float color) {
		this.color = color;
	}

	@Override
	public void dispose() {
		mesh.dispose();
		shader.dispose();
	}
	
	/**
	 * set the maximum radius
	 * @param r
	 */
	public void setR(int r) {
		this.r = r;
	}
	
	/**
	 * get the maximum radius
	 * @return
	 */
	public int getR() {
		return r;
	}
	
	public void setCamera(Camera camera) {
		this.camera = camera;
	}
	
	public Camera getCamera() {
		return camera;
	}
}
